classdef SegmentationLoss < dagnn.Loss

  properties
    inputNum = 1
  end

  methods
    function outputs = forward(obj, inputs, params)
      mass = sum(sum(inputs{end} > 0,2),1) + 1 ;
      for i = 1 : numel(inputs) - 1
          outputs{i} = vl_nnloss(inputs{i}, inputs{end}, [], ...
                             'loss', obj.loss, ...
                             'instanceWeights', 1./mass) ;
      end

      n = obj.numAveraged ;
      m = n + size(inputs{1},4) ;
      for j = 1 : numel(outputs)
        obj.average(j) = (n * obj.average(j) + double(gather(outputs{j}))) / m;
      end
      obj.numAveraged = m ;
    end

    function [derInputs, derParams] = backward(obj, inputs, params, derOutputs)
      mass = sum(sum(inputs{end} > 0,2),1) + 1 ;
      for i = 1 : numel(inputs) - 1
          derInputs{i} = vl_nnloss(inputs{i}, inputs{end}, derOutputs{i}, ...
                             'loss', obj.loss, ...
                             'instanceWeights', 1./mass) ;
      end

      derInputs{numel(inputs)} = [] ;
      derParams = {} ;
    end

    function obj = SegmentationLoss(varargin)
      obj.load(varargin) ;
      obj.average = zeros(obj.inputNum, 1);
    end
  end
end