function imdb = adeSetup(varargin)
opts.dataDir = fullfile('data/ade20k/ADEChallengeData2016') ;
opts.numGroups = 151; 
opts.includeSegmentation = true ;
opts.includeTest = false;
opts = vl_argparse(opts, varargin) ;

% Source images and classes
imdb.sets.id = uint8([1 2 3]) ;
imdb.sets.name = {'train', 'val', 'test'} ;
imdb.classes.id = uint8(1 : opts.numGroups - 1) ;
imdb.classes.name = {};
imdb.classes.images = cell(1, opts.numGroups - 1) ;
imdb.images.id = [] ;
imdb.images.name = {} ;
imdb.images.set = [] ;
index = containers.Map() ;
imdb.paths.image = esc(fullfile(opts.dataDir, 'images', '%s.jpg')) ;
imdb.paths.classSegmentation = esc(fullfile(opts.dataDir, 'annotations', '%s.png')) ;

[imdb, ~] = addImageSet(opts, imdb, index, 1) ;
[imdb, ~] = addImageSet(opts, imdb, index, 2) ;
if opts.includeTest
  [imdb, ~] = addImageSet(opts, imdb, index, 3) ;
end

% Compress data types
imdb.images.id = uint32(imdb.images.id) ;
imdb.images.set = uint8(imdb.images.set) ;
for i=1:opts.numGroups - 1
  imdb.classes.images{i} = uint32(imdb.classes.images{i}) ;
end

% Check images on disk and get their size
imdb = getImageSizes(imdb) ;

% -------------------------------------------------------------------------
function [imdb, index] = addImageSet(opts, imdb, index, sets)
% -------------------------------------------------------------------------
j = length(imdb.images.id) ;
if sets == 1
    segmentation_dir = fullfile(opts.dataDir, 'annotations/training');
elseif sets == 2
    segmentation_dir = fullfile(opts.dataDir, 'annotations/validation'); 
elseif sets == 3
    segmentation_dir = fullfile(opts.dataDir, 'annotations/test'); 
end
segmentations = dir(fullfile(segmentation_dir,'*.png'));
segmentationsName = {segmentations.name}';
imagesName = segmentationsName;
for i=1:length(imagesName)
  if ~index.isKey(imagesName{i})
    j = j + 1 ;
    index(imagesName{i}(1:end-4)) = j ;
    imdb.images.id(j) = j ;
    imdb.images.set(j) = sets ;
    imdb.images.name{j} = imagesName{i}(1:end-4) ;
    imdb.images.classification(j) = true ;
    imdb.images.segmentation(j) = true ;
  else
    j = index(imagesName{i}(1:end-4)) ;
  end
end

% -------------------------------------------------------------------------
function imdb = getImageSizes(imdb)
% -------------------------------------------------------------------------
for j=1:numel(imdb.images.id)
  info = imfinfo(sprintf(imdb.paths.image, imdb.images.name{j})) ;
  imdb.images.size(:,j) = uint16([info.Width ; info.Height]) ;
  fprintf('%s: checked image %s [%d x %d]\n', mfilename, imdb.images.name{j}, info.Height, info.Width) ;
end

% -------------------------------------------------------------------------
function str=esc(str)
% -------------------------------------------------------------------------
str = strrep(str, '\', '\\') ;
