function imdb = camvidSetup(varargin)
opts.dataDir = fullfile('data/CamVid') ;
opts.numGroups = 12; 
opts.includeSegmentation = true ;
opts.includeTest = true ;
opts = vl_argparse(opts, varargin) ;

% Source images and classes
imdb.paths.image = esc(fullfile(opts.dataDir, 'Images', '%s.png')) ;
imdb.sets.id = uint8([1 2 3]) ;
imdb.sets.name = {'train', 'val', 'test'} ;
imdb.classes.id = uint8(1:opts.numGroups-1) ;
imdb.classes.name = {'1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11'} ;
% imdb.classes.name = {'building', 'tree', 'sky', 'car', 'sign', 'road', 'pedestrian', 'fence', 'pole', 'sidewalk', 'bicyclist'} ;
imdb.classes.images = cell(1,opts.numGroups-1) ;
imdb.images.id = [] ;
imdb.images.name = {} ;
imdb.images.set = [] ;
index = containers.Map() ;
[imdb, index] = addImageSet(opts, imdb, index, 'train', 1) ;
[imdb, index] = addImageSet(opts, imdb, index, 'val', 2) ;
if opts.includeTest, [imdb, index] = addImageSet(opts, imdb, index, 'test', 3) ; end

% Source segmentations
if opts.includeSegmentation
  n = numel(imdb.images.id) ;
%   imdb.paths.objectSegmentation = esc(fullfile(opts.dataDir, 'SegmentationObject', '%s.png')) ;
  imdb.paths.classSegmentation = esc(fullfile(opts.dataDir, 'SegmentationClass', '%s.png')) ;
  imdb.images.segmentation = false(1, n) ;
  [imdb, index] = addSegmentationSet(opts, imdb, index, 'train', 1) ;
  [imdb, index] = addSegmentationSet(opts, imdb, index, 'val', 2) ;
  if opts.includeTest, [imdb, index] = addSegmentationSet(ohaolpts, imdb, index, 'test', 3) ; end
end

% Compress data types
imdb.images.id = uint32(imdb.images.id) ;
imdb.images.set = uint8(imdb.images.set) ;
for i=1:opts.numGroups-1
  imdb.classes.images{i} = uint32(imdb.classes.images{i}) ;
end

% Check images on disk and get their size
imdb = getImageSizes(imdb) ;

% -------------------------------------------------------------------------
function [imdb, index] = addImageSet(opts, imdb, index, setName, setCode)
% -------------------------------------------------------------------------
j = length(imdb.images.id) ;
image_dir = fullfile(opts.dataDir, setName);
images = dir(fullfile(image_dir,'*.png'));
imagesName = {images.name}';
for i=1:length(imagesName)
  if ~index.isKey(imagesName{i})
    j = j + 1 ;
    index(imagesName{i}(1:end-4)) = j ;
    imdb.images.id(j) = j ;
    imdb.images.set(j) = setCode ;
    imdb.images.name{j} = imagesName{i}(1:end-4) ;
    imdb.images.classification(j) = true ;
  else
    j = index(imagesName{i}(1:end-4)) ;
  end
end

% -------------------------------------------------------------------------
function [imdb, index] = addSegmentationSet(opts, imdb, index, setName, setCode)
% -------------------------------------------------------------------------
j = numel(imdb.images.id) ;
segmentation_dir = fullfile(opts.dataDir, setName);
segmentations = dir(fullfile(segmentation_dir,'*.png'));
segmentationsName = {segmentations.name}';
for i=1:length(segmentationsName)
  if index.isKey(segmentationsName{i}(1:end-4))
    k = index(segmentationsName{i}(1:end-4)) ;
    imdb.images.segmentation(k) = true ;
    imdb.images.set(k) = setCode ;
  else
    j = j + 1 ;
    index(segmentationsName{i}(1:end-4)) = j ;
    imdb.images.id(j) = j ;
    imdb.images.set(j) = setCode ;
    imdb.images.name{j} = segmentationsName{i}(1:end-4) ;
    imdb.images.classification(j) = false ;
    imdb.images.segmentation(j) = true ;
  end
end

% -------------------------------------------------------------------------
function imdb = getImageSizes(imdb)
% -------------------------------------------------------------------------
for j=1:numel(imdb.images.id)
  info = imfinfo(sprintf(imdb.paths.image, imdb.images.name{j})) ;
  imdb.images.size(:,j) = uint16([info.Width ; info.Height]) ;
  fprintf('%s: checked image %s [%d x %d]\n', mfilename, imdb.images.name{j}, info.Height, info.Width) ;
end

% -------------------------------------------------------------------------
function str=esc(str)
% -------------------------------------------------------------------------
str = strrep(str, '\', '\\') ;


