function net = fccnInitModel(varargin)
%FCNINITIALIZEMODEL Initialize the FCN-32 model from VGG-VD-16

opts.sourceModelPath = 'data/fcn32s-voc11/net-epoch-50.mat' ;
opts.numGroups = 21; 
opts = vl_argparse(opts, varargin) ;

if ~exist(opts.sourceModelPath)
  disp('No sourceModel!');
  quit;
end

net = load(opts.sourceModelPath) ;

net = dagnn.DagNN.loadobj(net.net) ;

%% Remove the last layer
net.removeLayer('deconv32') ;
net.removeLayer('objective') ;
net.removeLayer('accuracy') ;

% Modify the last fully-connected layer to have numGroups output classes or the same depth as pooling5, which is 512
% Initialize the new filters to zero
for i = [1 2]
  p = net.getParamIndex(net.layers(end).params{i}) ;
  if i == 1
    sz = size(net.params(p).value) ;
    sz(end) = 512 ;
  else
    sz = [512 1] ;
  end
  net.params(p).value = zeros(sz, 'single') ;
end
net.layers(end).block.size = size(...
  net.params(net.getParamIndex(net.layers(end).params{1})).value) ;

Poolings = [];
for l = 1:numel(net.layers)
  if strfind(net.layers(l).name, 'pool') == 1
    Poolings = [Poolings, net.layers(l).outputs];
  end
end

groupNum = [512 512 256 128 64 64];

for j = 1 : 5

  n = 6 - j;
  name = ['upsample', num2str(n)];
  upFilterName = [name, '_f'];
  in = 38 + 2 * (j-1);
  out = 38 + 2 * (j-1) + 1;
  inputs = ['x', num2str(in)];
  outputs = ['x', num2str(out)];

  upsampleName = ['upsample', num2str(6-j)];
  %% Add upsample layer
  filters = single(bilinear_u(4, groupNum(j), groupNum(j))) ;
  net.addLayer(name, ...
    dagnn.ConvTranspose('size', size(filters), ...
                        'upsample', 2, ...
                        'crop', 1, ...
                        'numGroups', groupNum(j), ...
                        'hasBias', false, ...
                        'opts', net.meta.cudnnOpts), ...
               inputs, upsampleName, upFilterName) ;

  f = net.getParamIndex(upFilterName) ;
  net.params(f).value = filters;
  net.params(f).learningRate = 0;
  net.params(f).weightDecay = 1;

  unpoolingName = ['unpooling', num2str(6-j)];
  unpoolingFilter = [unpoolingName, '_f'];
  filters = single(bilinear_u(4, groupNum(j), groupNum(j))) ;
  net.addLayer(unpoolingName, ...
    dagnn.ConvTranspose('size', size(filters), ...
                        'upsample', 2, ...
                        'crop', 1, ...
                        'numGroups', groupNum(j), ...
                        'hasBias', false, ...
                        'opts', net.meta.cudnnOpts), ...
               Poolings{6-j}, unpoolingName, unpoolingFilter) ;

  f = net.getParamIndex(unpoolingFilter) ;
  net.params(f).value = filters;
  net.params(f).learningRate = 0;
  net.params(f).weightDecay = 1;

  sumLayer = ['concat', num2str(6-j)];
  net.addLayer(sumLayer, dagnn.Concat(), {upsampleName, unpoolingName}, outputs) ;

  name = ['conv', num2str(n), '_D'];
  fileterName = [name, '_f'];
  biasName = [name, '_b'];
  in = 38 + 2 * (j-1) + 1;
  out = 38 + 2 * (j-1) + 2;
  inputs = ['x', num2str(in)];
  outputs = ['x', num2str(out)];

  % add convolution layer
  net.addLayer(name, ...
     dagnn.Conv('size', [3 3 groupNum(j) * 2 groupNum(j+1)], 'pad', 1), ...
     inputs, outputs, {fileterName, biasName});

  f = net.getParamIndex(fileterName) ;
  net.params(f).value = single(normrnd(0, 0.01, [3, 3, groupNum(j) * 2, groupNum(j+1)]));
  net.params(f).learningRate = 0.1;
  net.params(f).weightDecay = 1;

  b = net.getParamIndex(biasName) ;
  net.params(b).value = zeros(1, 1, groupNum(j+1), 'single') ;
  net.params(b).learningRate = 2;
  net.params(b).weightDecay = 1;

end

% add convolution layer
net.addLayer('predict5', ...
   dagnn.Conv('size', [1 1 groupNum(5) opts.numGroups]), ...
   'x48', 'x49', {'predict5_f', 'predict5_b'});

f = net.getParamIndex('predict5_f') ;
net.params(f).value = single(normrnd(0, 0.01, [1, 1, groupNum(5), opts.numGroups]));
net.params(f).learningRate = 0.1;
net.params(f).weightDecay = 1;

b = net.getParamIndex('predict5_b') ;
net.params(b).value = zeros(1, 1, opts.numGroups, 'single') ;
net.params(b).learningRate = 2;
net.params(b).weightDecay = 1;

%-------------------------------------------------------------------
net.addLayer('skip1', ...
     dagnn.Conv('size', [1 1 groupNum(1) opts.numGroups]), ...
     'x38', 'x50', {'skip1_f','skip1_b'});

f = net.getParamIndex('skip1_f') ;
net.params(f).value = single(normrnd(0, 0.01, [1, 1, groupNum(1), opts.numGroups])) ;
net.params(f).learningRate = 0.1 ;
net.params(f).weightDecay = 1 ;

f = net.getParamIndex('skip1_b') ;
net.params(f).value = zeros(1, 1, opts.numGroups, 'single') ;
net.params(f).learningRate = 2 ;
net.params(f).weightDecay = 1 ;

filters = single(bilinear_u(64, opts.numGroups, opts.numGroups)) ;
net.addLayer('predict1', ...
  dagnn.ConvTranspose('size', size(filters), ...
                      'upsample', 32, ...
                      'crop', 16, ...
                      'numGroups', opts.numGroups, ...
                      'hasBias', false, ...
                        'opts', net.meta.cudnnOpts), ...
                      'x50', 'x51', 'predict1f') ;

f = net.getParamIndex('predict1f') ;
net.params(f).value = filters ;
net.params(f).learningRate = 0 ;
net.params(f).weightDecay = 1 ;



%% Add the summation layer
net.addLayer('concat', dagnn.Concat(), {'x49', 'x51'}, 'x52') ;

net.addLayer('prediction', ...
     dagnn.Conv('size', [1 1 opts.numGroups * 2 opts.numGroups]), ...
     'x52', 'prediction', {'convfinal_f','convfinal_b'});

f = net.getParamIndex('convfinal_f') ;
net.params(f).value = single(normrnd(0, 0.01, [1, 1, opts.numGroups * 2, opts.numGroups])) ;
net.params(f).learningRate = 0.1 ;
net.params(f).weightDecay = 1 ;

f = net.getParamIndex('convfinal_b') ;
net.params(f).value = zeros(1, 1, opts.numGroups, 'single') ;
net.params(f).learningRate = 2 ;
net.params(f).weightDecay = 1 ;

% Add loss layer
net.addLayer('objective', ...
  SegmentationLoss('loss', 'softmaxlog'), ...
  {'prediction', 'label'}, 'objective') ;

% Add accuracy layer
net.addLayer('accuracy', ...
  SegmentationAccuracy('numGroups', opts.numGroups), ...
  {'prediction', 'label'}, 'accuracy') ;
