function net = fccnChDataModel(varargin)

opts.sourceModelPath = 'data/fccn8-laneset/net-epoch-100.mat' ;
opts.numGroups = 5; 
opts = vl_argparse(opts, varargin) ;

if ~exist(opts.sourceModelPath)
  disp('No sourceModel!');
  quit;
end

net = load(opts.sourceModelPath) ;

net = dagnn.DagNN.loadobj(net.net) ;

%% Remove the last layer
net.removeLayer('objective') ;
net.removeLayer('accuracy') ;
net.removeLayer('prediction');
net.removeLayer('concat');
net.removeLayer('predict5');
net.removeLayer('skip4');
net.removeLayer('predict4');
net.removeLayer('skip3');
net.removeLayer('predict3');
net.removeLayer('skip2');
net.removeLayer('predict2');
net.removeLayer('skip1');
net.removeLayer('predict1');


groupNum = [512 512 256 128 64 64];

% add convolution layer
net.addLayer('predict5', ...
   dagnn.Conv('size', [1 1 groupNum(5) opts.numGroups]), ...
   'x48', 'x49', {'predict5_f', 'predict5_b'});

f = net.getParamIndex('predict5_f') ;
net.params(f).value = single(normrnd(0, 0.01, [1, 1, groupNum(5), opts.numGroups]));
net.params(f).learningRate = 0.1;
net.params(f).weightDecay = 1;

b = net.getParamIndex('predict5_b') ;
net.params(b).value = zeros(1, 1, opts.numGroups, 'single') ;
net.params(b).learningRate = 2;
net.params(b).weightDecay = 1;

%-------------------------------------------------------------------
net.addLayer('skip4', ...
     dagnn.Conv('size', [1 1 groupNum(4) opts.numGroups]), ...
     'x44', 'x50', {'skip4_f','skip4_b'});

f = net.getParamIndex('skip4_f') ;
net.params(f).value = single(normrnd(0, 0.01, [1, 1, groupNum(4), opts.numGroups])) ;
net.params(f).learningRate = 0.1 ;
net.params(f).weightDecay = 1 ;

f = net.getParamIndex('skip4_b') ;
net.params(f).value = zeros(1, 1, opts.numGroups, 'single') ;
net.params(f).learningRate = 2 ;
net.params(f).weightDecay = 1 ;

filters = single(bilinear_u(8, opts.numGroups, opts.numGroups)) ;
net.addLayer('predict4', ...
  dagnn.ConvTranspose('size', size(filters), ...
                      'upsample', 4, ...
                      'crop', 2, ...
                      'numGroups', opts.numGroups, ...
                      'hasBias', false, ...
                      'opts', net.meta.cudnnOpts), ...
             'x50', 'x51', 'predict4f') ;

f = net.getParamIndex('predict4f') ;
net.params(f).value = filters ;
net.params(f).learningRate = 0 ;
net.params(f).weightDecay = 1 ;

%-------------------------------------------------------------------
net.addLayer('skip3', ...
     dagnn.Conv('size', [1 1 groupNum(3) opts.numGroups]), ...
     'x42', 'x52', {'skip3_f','skip3_b'});

f = net.getParamIndex('skip3_f') ;
net.params(f).value = single(normrnd(0, 0.01, [1, 1, groupNum(3), opts.numGroups])) ;
net.params(f).learningRate = 0.1 ;
net.params(f).weightDecay = 1 ;

f = net.getParamIndex('skip3_b') ;
net.params(f).value = zeros(1, 1, opts.numGroups, 'single') ;
net.params(f).learningRate = 2 ;
net.params(f).weightDecay = 1 ;

filters = single(bilinear_u(16, opts.numGroups, opts.numGroups)) ;
net.addLayer('predict3', ...
  dagnn.ConvTranspose('size', size(filters), ...
                      'upsample', 8, ...
                      'crop', 4, ...
                      'numGroups', opts.numGroups, ...
                      'hasBias', false, ...
                      'opts', net.meta.cudnnOpts), ...
             'x52', 'x53', 'predict3f') ;

f = net.getParamIndex('predict3f') ;
net.params(f).value = filters ;
net.params(f).learningRate = 0 ;
net.params(f).weightDecay = 1 ;

%-------------------------------------------------------------------
net.addLayer('skip2', ...
     dagnn.Conv('size', [1 1 groupNum(2) opts.numGroups]), ...
     'x40', 'x54', {'skip2_f','skip2_b'});

f = net.getParamIndex('skip2_f') ;
net.params(f).value = single(normrnd(0, 0.01, [1, 1, groupNum(2), opts.numGroups])) ;
net.params(f).learningRate = 0.1 ;
net.params(f).weightDecay = 1 ;

f = net.getParamIndex('skip2_b') ;
net.params(f).value = zeros(1, 1, opts.numGroups, 'single') ;
net.params(f).learningRate = 2 ;
net.params(f).weightDecay = 1 ;

filters = single(bilinear_u(32, opts.numGroups, opts.numGroups)) ;
net.addLayer('predict2', ...
  dagnn.ConvTranspose('size', size(filters), ...
                      'upsample', 16, ...
                      'crop', 8, ...
                      'numGroups', opts.numGroups, ...
                      'hasBias', false, ...
                      'opts', net.meta.cudnnOpts), ...
             'x54', 'x55', 'predict2f') ;

f = net.getParamIndex('predict2f') ;
net.params(f).value = filters ;
net.params(f).learningRate = 0 ;
net.params(f).weightDecay = 1 ;

%-------------------------------------------------------------------
net.addLayer('skip1', ...
     dagnn.Conv('size', [1 1 groupNum(1) opts.numGroups]), ...
     'x38', 'x56', {'skip1_f','skip1_b'});

f = net.getParamIndex('skip1_f') ;
net.params(f).value = single(normrnd(0, 0.01, [1, 1, groupNum(1), opts.numGroups])) ;
net.params(f).learningRate = 0.1 ;
net.params(f).weightDecay = 1 ;

f = net.getParamIndex('skip1_b') ;
net.params(f).value = zeros(1, 1, opts.numGroups, 'single') ;
net.params(f).learningRate = 2 ;
net.params(f).weightDecay = 1 ;

filters = single(bilinear_u(64, opts.numGroups, opts.numGroups)) ;
net.addLayer('predict1', ...
  dagnn.ConvTranspose('size', size(filters), ...
                      'upsample', 32, ...
                      'crop', 16, ...
                      'numGroups', opts.numGroups, ...
                      'hasBias', false, ...
                        'opts', net.meta.cudnnOpts), ...
                      'x56', 'x57', 'predict1f') ;

f = net.getParamIndex('predict1f') ;
net.params(f).value = filters ;
net.params(f).learningRate = 0 ;
net.params(f).weightDecay = 1 ;



%% Add the summation layer
net.addLayer('concat', dagnn.Concat(), {'x49', 'x51', 'x53', 'x55', 'x57'}, 'x58') ;

net.addLayer('prediction', ...
     dagnn.Conv('size', [1 1 opts.numGroups * 5 opts.numGroups]), ...
     'x58', 'prediction', {'convfinal_f','convfinal_b'});

f = net.getParamIndex('convfinal_f') ;
net.params(f).value = single(normrnd(0, 0.01, [1, 1, opts.numGroups * 5, opts.numGroups])) ;
net.params(f).learningRate = 0.1 ;
net.params(f).weightDecay = 1 ;

f = net.getParamIndex('convfinal_b') ;
net.params(f).value = zeros(1, 1, opts.numGroups, 'single') ;
net.params(f).learningRate = 2 ;
net.params(f).weightDecay = 1 ;

% Add loss layer
net.addLayer('objective', ...
  SegmentationLoss('loss', 'softmaxlog', 'inputNum', 6), ...
  {'prediction', 'x49', 'x51', 'x53', 'x55', 'x57', 'label'}, {'objective', 'Loss5', 'Loss4', 'Loss3', 'Loss2', 'Loss1'}) ;

% Add accuracy layer
net.addLayer('accuracy', ...
  SegmentationAccuracy('numGroups', opts.numGroups), ...
  {'prediction', 'label'}, 'accuracy') ;
