function fccnTrain(varargin)

% addpath(genpath('/home/autolab/Documents/matconvnet-master'));

% run vl_setupnn ;

% experiment and data paths
opts.expDir = 'data/fccn8-camvid' ;
opts.dataDir = 'data/CamVid' ;

opts.numGroups = 12;    % add by yangtao 2015.04.25

opts.sourceModelPath = 'data/fcn32s-camvid/net-epoch-50.mat' ;
[opts, varargin] = vl_argparse(opts, varargin) ;

% experiment setup
opts.imdbPath = fullfile(opts.dataDir, 'imdb.mat') ;
opts.imdbStatsPath = fullfile(opts.dataDir, 'imdbStats.mat') ;

opts.numFetchThreads = 1 ; % not used yet

% training options (SGD)
opts.train = [];
[opts, varargin] = vl_argparse(opts, varargin) ;

trainOpts.batchSize = 20 ;
trainOpts.numSubBatches = 10 ;
trainOpts.continue = true ;
trainOpts.gpus = [2] ;
trainOpts.prefetch = true ;
trainOpts.expDir = opts.expDir ;
trainOpts.numGroups = opts.numGroups;
% lr = ones(1,50);
% lr(1:10) = 0.001 * lr(1:10);
% lr(11:20) = 0.0005 * lr(11:20);
% lr(21:30) = 0.0001 * lr(21:30);
% lr(31:40) = 0.00005 * lr(31:40);
% lr(41:50) = 0.00001 * lr(41:50);
trainOpts.learningRate = 0.0001 * ones(1,100) ;
trainOpts.numEpochs = numel(trainOpts.learningRate) ;

% -------------------------------------------------------------------------
% Setup data
% -------------------------------------------------------------------------

% Get PASCAL VOC 12 segmentation dataset plus Berkeley's additional
% segmentations
if exist(opts.imdbPath)
  imdb = load(opts.imdbPath) ;
else
  imdb = camvidSetup('dataDir', opts.dataDir, ...
    'numGroups', opts.numGroups, ...
    'includeTest', true, ...
    'includeSegmentation', true) ;
  mkdir(opts.expDir) ;
  save(opts.imdbPath, '-struct', 'imdb') ;
end

% Get training and test/validation subsets
% train = find(imdb.images.set == 1 & imdb.images.segmentation) ;
train = find(imdb.images.segmentation);
val = find(imdb.images.set == 2 & imdb.images.segmentation) ;

% Get dataset statistics
if exist(opts.imdbStatsPath)
  stats = load(opts.imdbStatsPath) ;
else
  stats = getDatasetStatistics(imdb, opts.numGroups) ;
  save(opts.imdbStatsPath, '-struct', 'stats') ;
end

% -------------------------------------------------------------------------
% Setup model
% -------------------------------------------------------------------------

% Get initial model from VGG-VD-16
net = fccnInitModel3('sourceModelPath', opts.sourceModelPath, 'numGroups', opts.numGroups) ;
% net = chDataModel(opts.sourceModelPath, opts.numGroups);

% net = load(opts.sourceModelPath) ;
% net = dagnn.DagNN.loadobj(net.net) ;

net.meta.normalization.rgbMean = stats.rgbMean ;
net.meta.classes = imdb.classes.name ;

% -------------------------------------------------------------------------
% Train
% -------------------------------------------------------------------------

% Setup data fetching options
bopts.numThreads = opts.numFetchThreads ;
bopts.labelStride = 1 ;
bopts.labelOffset = 1 ;
bopts.classWeights = ones(1,opts.numGroups,'single') ;
bopts.rgbMean = stats.rgbMean ;
bopts.useGpu = numel(trainOpts.gpus) > 0 ;


lossWeights = [1.0 1.0 1.0 1.0 1.0 1.0];  % fccn8
% lossWeights = [0.5 0.5 0.5 0.5 0.5];  % fccn9
% lossWeights = [0.2 0.2 0.2 0.2 0.2 1.0];  % fccn10
% lossWeights = [0.1 0.1 0.1 0.1 0.1 1.0];  % fccn11
% lossWeights = [0 0 0 0 0 1.0];  % fccn12
% lossWeights = [1.0 0 0 0 0 1.0];  % fccn13
% lossWeights = [0 1.0 0 0 0];  % fccn14
% lossWeights = [0 0 1.0 0 0];  % fccn15
% lossWeights = [0 0 0 1.0 0];  % fccn16
% lossWeights = [0 0 0 0 1.0];  % fccn17
% lossWeights = [0.1 0.1 0.25 0.5 0.75 1.0];  % fccn20
% lossWeights = [1.0 0 0 0 0 0];  % fccn21
% lossWeights = [0 0 1.0 0 0 0];  % fccn21
% lossWeights = [0.9 0.7 0.5 0.3 0.1 1.0];  % fccn26
% lossWeights = [0.5 0.5 0 0 0 1.0];  % fccn27
% lossWeights = [0.33 0.33 0.33 0 0 1.0];  % fccn28
% lossWeights = [0.25 0.25 0.25 0.25 0 1.0];  % fccn29
% lossWeights = [0.1 0.3 0.5 0.7 0.9 1.0];  % fccn30

derOutputs = {'Loss1', lossWeights(1), ...
              'Loss2', lossWeights(2), ...
              'Loss3', lossWeights(3), ...
              'Loss4', lossWeights(4), ...
              'Loss5', lossWeights(5), ...
              'objective', lossWeights(6)};
% derOutputs = {'objective', 1};

% Launch SGD
info = cnn_train_dag(net, imdb, getBatchWrapper(bopts), ...
                     trainOpts, ....
                     'train', train, ...
                     'val', val, ...
                     'derOutputs', derOutputs) ;

% trainOpts.expDir = 'data/fccn9-camvid' ;
% trainOpts.learningRate = 0.0001 * ones(1,50) ;
% lossWeights = [0.5 0.5 0.5 0.5 0.5 1.0];  % fccn9
% derOutputs = {'Loss1', lossWeights(1), ...
%               'Loss2', lossWeights(2), ...
%               'Loss3', lossWeights(3), ...
%               'Loss4', lossWeights(4), ...
%               'Loss5', lossWeights(5), ...
%               'objective', lossWeights(6)};
% % derOutputs = {'objective', 1};
% info = cnn_train_dag(net, imdb, getBatchWrapper(bopts), ...
%                      trainOpts, ....
%                      'train', train, ...
%                      'val', val, ...
%                      'derOutputs', derOutputs) ;

% -------------------------------------------------------------------------
function fn = getBatchWrapper(opts)
% -------------------------------------------------------------------------
fn = @(imdb,batch) getBatch(imdb,batch,opts,'prefetch',nargout==0) ;
