function net = fcnBNModel(varargin)
% add batch normalization layer

opts.sourceModelPath = 'data/fcn32s-voc11/net-epoch-50.mat' ;
opts.numGroups = 21; 
% opts = vl_argparse(opts, varargin) ;

if ~exist(opts.sourceModelPath)
  disp('No sourceModel!');
  quit;
end

net = load(opts.sourceModelPath) ;
net = dagnn.DagNN.loadobj(net.net);

convIndex = [];
for l = 1:numel(net.layers)
	if strfind(net.layers(l).name, 'conv') == 1
        convIndex = [convIndex, l];
    elseif strfind(net.layers(l).name, 'fc') == 1
		convIndex = [convIndex, l];
	end
end


if convIndex
	for i = 1:numel(convIndex)
		layerName = [net.layers(convIndex(i)).name, '_bn'];
		outputs = ['bn', num2str(i)];
        paramf = [outputs, 'f',];
        paramb = [outputs, 'b',];
        paramm = [outputs, 'm',];
		numChannels = net.layers(convIndex(i)).block.size(4);
		net.addLayer(layerName,	...
            dagnn.BatchNorm('numChannels', numChannels), ...
            net.layers(convIndex(i)).outputs, ...
            outputs,{paramf, paramb, paramm});
		net.layers(convIndex(i)+1).inputs = outputs;
        
        f = net.getParamIndex(paramf);
        net.params(f).value = ones(numChannels, 1, 'single');
        net.params(f).learningRate = 2;
        net.params(f).weightDecay = 0;

        b = net.getParamIndex(paramb);
        net.params(b).value = zeros(numChannels, 1, 'single');
        net.params(b).learningRate = 1;
        net.params(b).weightDecay = 0;

        m = net.getParamIndex(paramm);
        net.params(m).value = zeros(numChannels, 2, 'single');
        net.params(m).learningRate = 0.05;
        net.params(m).weightDecay = 0;
	end
end

net.rebuild();