function net = fcnDeconvAllInitModel(varargin)
%FCNINITIALIZEMODEL Initialize the FCN-32 model from VGG-VD-16

opts.sourceModelPath = 'data/fcn32s-voc11/net-epoch-50.mat' ;
opts.numGroups = 21; 
opts = vl_argparse(opts, varargin) ;

if ~exist(opts.sourceModelPath)
  disp('No sourceModel!');
  quit;
end

net = load(opts.sourceModelPath) ;

net = dagnn.DagNN.loadobj(net.net) ;

%% Remove the last layer
net.removeLayer('deconv32') ;
net.removeLayer('objective') ;
net.removeLayer('accuracy') ;

% Modify the last fully-connected layer to have numGroups output classes
% Initialize the new filters to zero
for i = [1 2]
  p = net.getParamIndex(net.layers(end).params{i}) ;
  if i == 1
    sz = size(net.params(p).value) ;
    sz(end) = opts.numGroups ;
  else
    sz = [opts.numGroups 1] ;
  end
  net.params(p).value = zeros(sz, 'single') ;
end
net.layers(end).block.size = size(...
  net.params(net.getParamIndex(net.layers(end).params{1})).value) ;

for i = 1 : 2

  n = 8 - i;
  name = ['fc', num2str(n), '_D'];
  fileterName = [name, '_f'];
  biasName = [name, '_b'];
  in = 38 + 3 * (i-1);
  out = 38 + 3 * (i-1) + 1;
  inputs = ['x', num2str(in)];
  outputs = ['x', num2str(out)];

  groupNum = 0;
  if i == 1
    groupNum = opts.numGroups;
  else
    groupNum = 4096;
  end
  % add convolution layer
  net.addLayer(name, ...
     dagnn.Conv('size', [1 1 groupNum 4096], 'pad', 0), ...
     inputs, outputs, {fileterName, biasName});

  f = net.getParamIndex(fileterName) ;
  net.params(f).value = single(normrnd(0, 0.01, [1, 1, groupNum, 4096]));
  net.params(f).learningRate = 1;
  net.params(f).weightDecay = 1;

  b = net.getParamIndex(biasName) ;
  net.params(b).value = zeros(1, 1, 4096, 'single') ;
  net.params(b).learningRate = 2;
  net.params(b).weightDecay = 1;

  name = ['relu', num2str(n), '_D'];
  out = 38 + 3 * (i-1) + 2;
  inputs = outputs;
  outputs = ['x', num2str(out)];
  % add relu layer
  net.addLayer(name, ...
     dagnn.ReLU('useShortCircuit', 1, 'leak', 0), ...
     inputs, outputs, {});

  name = ['dropout', num2str(n), '_D'];
  out = 38 + 3 * (i-1) + 3;
  inputs = outputs;
  outputs = ['x', num2str(out)];
  % add dropout layer
  net.addLayer(name, ...
     dagnn.DropOut('rate', 0.5, 'frozen', false), ...
     inputs, outputs, {});

end

groupNum = [512 512 256 128 64];

for j = 1 : 5

  n = 6 - j;
  name = ['upsample', num2str(n)];
  upFilterName = [name, '_f'];
  in = 44 + 4 * (j-1);
  out = 44 + 4 * (j-1) + 1;
  inputs = ['x', num2str(in)];
  outputs = ['x', num2str(out)];

  inputNum = 0;
  if j == 1
    inputNum = 4096;
  else
    inputNum = groupNum(j-1);
  end
  %% Add upsample layer
  filters = single(bilinear_u(4, inputNum, inputNum)) ;
  net.addLayer(name, ...
    dagnn.ConvTranspose('size', size(filters), ...
                        'upsample', 2, ...
                        'crop', 1, ...
                        'numGroups', inputNum, ...
                        'hasBias', false, ...
                        'opts', net.meta.cudnnOpts), ...
               inputs, outputs, upFilterName) ;

  f = net.getParamIndex(upFilterName) ;
  net.params(f).value = filters;
  net.params(f).learningRate = 0;
  net.params(f).weightDecay = 1;

  name = ['conv', num2str(n), '_D'];
  fileterName = [name, '_f'];
  biasName = [name, '_b'];
  out = 44 + 4 * (j-1) + 2;
  inputs = outputs;
  outputs = ['x', num2str(out)];

  % add convolution layer
  net.addLayer(name, ...
     dagnn.Conv('size', [3 3 inputNum groupNum(j)], 'pad', 1), ...
     inputs, outputs, {fileterName, biasName});

  f = net.getParamIndex(fileterName) ;
  net.params(f).value = single(normrnd(0, 0.01, [3, 3, inputNum, groupNum(j)]));
  net.params(f).learningRate = 1;
  net.params(f).weightDecay = 1;

  b = net.getParamIndex(biasName) ;
  net.params(b).value = zeros(1, 1, groupNum(j), 'single') ;
  net.params(b).learningRate = 2;
  net.params(b).weightDecay = 1;

  name = ['sum', num2str(n)];
  f = find(strcmp(['conv', num2str(n), '_2'], {net.layers.name}));
  inputs = [outputs, net.layers(f).outputs];
  out = 44 + 4 * (j-1) + 3;
  outputs = ['x', num2str(out)];
  %% Add the summation layer
  net.addLayer(name, dagnn.Sum(), inputs, outputs) ;

  name = ['relu', num2str(n), '_D'];
  out = 44 + 4 * (j-1) + 4;
  inputs = outputs;
  outputs = ['x', num2str(out)];
  % add relu layer
  net.addLayer(name, ...
     dagnn.ReLU('useShortCircuit', 1, 'leak', 0), ...
     inputs, outputs, {});

end

% add convolution layer
net.addLayer('convfinal', ...
   dagnn.Conv('size', [3 3 groupNum(5) opts.numGroups], 'pad', 1), ...
   'x64', 'x65', {'convf_f', 'convf_b'});

f = net.getParamIndex('convf_f') ;
net.params(f).value = zeros(3, 3, groupNum(5), opts.numGroups, 'single');
net.params(f).learningRate = 0.1;
net.params(f).weightDecay = 1;

b = net.getParamIndex('convf_b') ;
net.params(b).value = zeros(1, 1, opts.numGroups, 'single') ;
net.params(b).learningRate = 2;
net.params(b).weightDecay = 1;

filters = single(bilinear_u(64, opts.numGroups, opts.numGroups)) ;
net.addLayer('deconv32', ...
  dagnn.ConvTranspose(...
  'size', size(filters), ...
  'upsample', 32, ...
  'crop', [16 16 16 16], ...
  'numGroups', opts.numGroups, ...
  'hasBias', false, ...
    'opts', net.meta.cudnnOpts), ...
  'x38', 'x66', 'deconv32f') ;

f = net.getParamIndex('deconv32f') ;
net.params(f).value = filters ;
net.params(f).learningRate = 0 ;
net.params(f).weightDecay = 1 ;

%% Add the summation layer
net.addLayer('sumfinal', dagnn.Sum(), {'x65', 'x66'}, 'prediction') ;

% Modify the bias learning rate for all layers
for i = 1:numel(net.layers)-1
  if (isa(net.layers(i).block, 'dagnn.Conv') && net.layers(i).block.hasBias)
    filt = net.getParamIndex(net.layers(i).params{1}) ;
    bias = net.getParamIndex(net.layers(i).params{2}) ;
    net.params(bias).learningRate = 2 * net.params(filt).learningRate ;
  end
end

% Add loss layer
net.addLayer('objective', ...
  SegmentationLoss('loss', 'softmaxlog'), ...
  {'prediction', 'label'}, 'objective') ;

% Add accuracy layer
net.addLayer('accuracy', ...
  SegmentationAccuracy(), ...
  {'prediction', 'label'}, 'accuracy') ;

if 0
  figure(100) ; clf ;
  n = numel(net.vars) ;
  for i=1:n
    vl_tightsubplot(n,i) ;
    showRF(net, 'input', net.vars(i).name) ;
    title(sprintf('%s', net.vars(i).name)) ;
    drawnow ;
  end
end