function fcnTrain(varargin)
%FNCTRAIN Train FCN model using MatConvNet

% addpath(genpath('/home/autolab/Documents/matconvnet-master'));

% run vl_setupnn ;
% addpath matconvnet/examples ;

% experiment and data paths
opts.expDir = 'data/fcn32sDeconvPlus1-panolanepark' ;
opts.dataDir = 'data/panorama/panoLanePark' ;
% opts.modelType = 'fcn8s' ;

opts.numGroups = 6;    % add by yangtao 2015.04.25

opts.sourceModelPath = 'data/fcn32s-camvid/net-epoch-50.mat' ;
% opts.sourceModelPath = 'data/models/imagenet-vgg-verydeep-16.mat';
[opts, varargin] = vl_argparse(opts, varargin) ;

% experiment setup
opts.imdbPath = fullfile(opts.dataDir, 'imdb.mat') ;
opts.imdbStatsPath = fullfile(opts.dataDir, 'imdbStats.mat') ;
opts.vocEdition = '11' ;
opts.vocAdditionalSegmentations = true ;

opts.numFetchThreads = 1 ; % not used yet

% training options (SGD)
opts.train = [] ;
[opts, varargin] = vl_argparse(opts, varargin) ;

opts.train.batchSize = 20 ;
opts.train.numSubBatches = 10 ;
opts.train.continue = true ;
opts.train.gpus = [1] ;
opts.train.prefetch = true ;
opts.train.expDir = opts.expDir ;
% lr = ones(1,50);
% lr(1:10) = 0.001 * lr(1:10);
% lr(11:20) = 0.0005 * lr(11:20);
% lr(21:30) = 0.0001 * lr(21:30);
% lr(31:40) = 0.00005 * lr(31:40);
% lr(41:50) = 0.00001 * lr(41:50);
opts.train.learningRate = 0.0001 * ones(1,50) ;
opts.train.numEpochs = numel(opts.train.learningRate) ;

% -------------------------------------------------------------------------
% Setup data
% -------------------------------------------------------------------------

% Get PASCAL VOC 12 segmentation dataset plus Berkeley's additional
% segmentations
if exist(opts.imdbPath)
  imdb = load(opts.imdbPath) ;
else
%   imdb = vocSetup('dataDir', opts.dataDir, ...
%     'numGroups', opts.numGroups, ...
%     'edition', opts.vocEdition, ...
%     'includeTest', false, ...
%     'includeSegmentation', true, ...
%     'includeDetection', false) ;
%   if opts.vocAdditionalSegmentations
%     imdb = vocSetupAdditionalSegmentations(imdb, 'dataDir', opts.dataDir, 'numGroups', opts.numGroups) ;
%   end
  imdb = camvidSetup('dataDir', opts.dataDir, ...
    'numGroups', opts.numGroups, ...
    'includeTest', true, ...
    'includeSegmentation', true) ;
  mkdir(opts.expDir) ;
  save(opts.imdbPath, '-struct', 'imdb') ;
end

% Get training and test/validation subsets
train = find(imdb.images.set == 1 & imdb.images.segmentation) ;
val = find(imdb.images.set == 2 & imdb.images.segmentation) ;

% Get dataset statistics
if exist(opts.imdbStatsPath)
  stats = load(opts.imdbStatsPath) ;
else
  stats = getDatasetStatistics(imdb) ;
  save(opts.imdbStatsPath, '-struct', 'stats') ;
end

% -------------------------------------------------------------------------
% Setup model
% -------------------------------------------------------------------------

% Get initial model from VGG-VD-16
net = fcnDeconvPlus1InitModel('sourceModelPath', opts.sourceModelPath, 'numGroups', opts.numGroups) ;
% net = fcnInitializeModel('sourceModelPath', opts.sourceModelPath, 'numGroups', opts.numGroups) ;
% if any(strcmp(opts.modelType, {'fcn16s', 'fcn8s', 'fcn4s'}))
%   % upgrade model to FCN16s
%   net = fcnInitializeModel16s(net, opts.numGroups) ;
% end
% if any(strcmp(opts.modelType, {'fcn8s', 'fcn4s'}))
%   % upgrade model fto FCN8s
%   net = fcnInitializeModel8s(net, opts.numGroups) ;
% end
% if strcmp(opts.modelType, 'fcn4s')
%   % upgrade model fto FCN8s
%   net = fcnInitializeModel4s(net, opts.numGroups) ;
% end

% net = fcnBNModel('sourceModelPath', opts.sourceModelPath, 'numGroups', opts.numGroups);

% net = load(opts.sourceModelPath) ;
% net = dagnn.DagNN.loadobj(net.net) ;

net.meta.normalization.rgbMean = stats.rgbMean ;
net.meta.classes = imdb.classes.name ;

% -------------------------------------------------------------------------
% Train
% -------------------------------------------------------------------------

% Setup data fetching options
bopts.numThreads = opts.numFetchThreads ;
bopts.labelStride = 1 ;
bopts.labelOffset = 1 ;
bopts.classWeights = ones(1,opts.numGroups,'single') ;
bopts.rgbMean = stats.rgbMean ;
bopts.useGpu = numel(opts.train.gpus) > 0 ;

% Launch SGD
info = cnn_train_dag(net, imdb, getBatchWrapper(bopts), ...
                     opts.train, ....
                     'train', train, ...
                     'val', val, ...
                     opts.train) ;

% -------------------------------------------------------------------------
function fn = getBatchWrapper(opts)
% -------------------------------------------------------------------------
fn = @(imdb,batch) getBatch(imdb,batch,opts,'prefetch',nargout==0) ;
