function [net,stats] = cnn_train_dag1(net, imdb, getBatch, varargin)
%CNN_TRAIN_DAG Demonstrates training a CNN using the DagNN wrapper
%    CNN_TRAIN_DAG() is similar to CNN_TRAIN(), but works with
%    the DagNN wrapper instead of the SimpleNN wrapper.

% Copyright (C) 2014-15 Andrea Vedaldi.
% All rights reserved.
%
% This file is part of the VLFeat library and is made available under
% the terms of the BSD license (see the COPYING file).

% Todo: save momentum with checkpointing (a waste?)

opts.expDir = fullfile('data','exp') ;
opts.continue = true ;
opts.batchSize = 256 ;
opts.numSubBatches = 1 ;
opts.train = [] ;
opts.val = [] ;
opts.gpus = [] ;
opts.prefetch = false ;
opts.numEpochs = 300 ;
opts.learningRate = 0.001 ;
opts.weightDecay = 0.0005 ;
opts.momentum = 0.9 ;
opts.memoryMapFile = fullfile(tempdir, 'matconvnet.bin') ;
opts.profile = false ;

opts.numGroups = 21; % add by yangtao 2016.12.28

opts.derOutputs = {'objective', 1} ;
opts.extractStatsFn = @extractStats ;
opts.plotStatistics = true;
opts = vl_argparse(opts, varargin) ;

if ~exist(opts.expDir, 'dir'), mkdir(opts.expDir) ; end
if isempty(opts.train), opts.train = find(imdb.images.set==1) ; end
if isempty(opts.val), opts.val = find(imdb.images.set==2) ; end
if isnan(opts.train), opts.train = [] ; end

% -------------------------------------------------------------------------
%                                                            Initialization
% -------------------------------------------------------------------------

state.getBatch = getBatch ;
evaluateMode = isempty(opts.train) ;
if ~evaluateMode
  if isempty(opts.derOutputs)
    error('DEROUTPUTS must be specified when training.\n') ;
  end
end
stats = [] ;

% setup GPUs
numGpus = numel(opts.gpus) ;
if numGpus > 1
  if isempty(gcp('nocreate')),
    parpool('local',numGpus) ;
    spmd, gpuDevice(opts.gpus(labindex)), end  %多线程貌似失败
  end
  if exist(opts.memoryMapFile)
    delete(opts.memoryMapFile) ;
  end
elseif numGpus == 1
  gpuDevice(opts.gpus)
end

% -------------------------------------------------------------------------
%                                                        Train and validate
% -------------------------------------------------------------------------

modelPath = @(ep) fullfile(opts.expDir, sprintf('net-epoch-%d.mat', ep));
modelFigPath = fullfile(opts.expDir, 'net-train.ps') ;

% add by yangtao 2017.01.10
segAccIndex = numel(net.layers);
for i = 1 : numel(net.layers)
  if ~isempty(strfind(net.layers(i).name, 'accuracy'))
    segAccIndex = i;
  end
end

start = opts.continue * findLastCheckpoint(opts.expDir) ;
if start >= 1
  fprintf('%s: resuming by loading epoch %d\n', mfilename, start) ;
  [net, stats] = loadState(modelPath(start)) ;
  net.layers(segAccIndex).block.numGroups = opts.numGroups;
end

for epoch=start+1:opts.numEpochs

  if mod(epoch, 50) == 1

    net.removeLayer('objective') ;
    net.removeLayer('accuracy') ;

    % lo = fix(epoch / 50);

    if epoch > 1
      for i = 1:38
        if (isa(net.layers(i).block, 'dagnn.Conv') && net.layers(i).block.hasBias)
          filt = net.getParamIndex(net.layers(i).params{1}) ;
          bias = net.getParamIndex(net.layers(i).params{2}) ;
          net.params(bias).learningRate = net.params(bias).learningRate / 100 ;
          net.params(filt).learningRate = net.params(filt).learningRate / 100 ;
        end
      end
    end

    switch epoch
      case 251
        
        net.params(64).learningRate = 0.0001;
        net.params(65).learningRate = 0.0002;

        opts.derOutputs = { 'Loss1', 1, ...
                            'Loss2', 0, ...
                            'Loss3', 0, ...
                            'Loss4', 0, ...
                            'Loss5', 0, ...
                            'objective', 0};
        net.addLayer('objective', ...
          SegmentationLoss('loss', 'softmaxlog', 'inputNum', 6), ...
          {'x57', 'prediction', 'x49', 'x51', 'x53', 'x55', 'label'}, {'Loss1', 'objective', 'Loss5', 'Loss4', 'Loss3', 'Loss2'}) ;

        net.addLayer('accuracy', ...
          SegmentationAccuracy('numGroups', opts.numGroups), ...
          {'x57', 'label'}, 'accuracy') ;

      case 201

        net.params(61).learningRate = 0.0001;
        net.params(62).learningRate = 0.0002;
        net.params(35).learningRate = 0.0001;
        net.params(36).learningRate = 0.0002;

        opts.derOutputs = { 'Loss1', 0, ...
                            'Loss2', 1, ...
                            'Loss3', 0, ...
                            'Loss4', 0, ...
                            'Loss5', 0, ...
                            'objective', 0};
                            
        net.addLayer('objective', ...
          SegmentationLoss('loss', 'softmaxlog', 'inputNum', 6), ...
          {'x55', 'x57', 'prediction', 'x49', 'x51', 'x53', 'label'}, {'Loss2', 'Loss1', 'objective', 'Loss5', 'Loss4', 'Loss3'}) ;

        net.addLayer('accuracy', ...
          SegmentationAccuracy('numGroups', opts.numGroups), ...
          {'x55', 'label'}, 'accuracy') ;

      case 151

        net.params(58).learningRate = 0.0001;
        net.params(59).learningRate = 0.0002;
        net.params(39).learningRate = 0.0001;
        net.params(40).learningRate = 0.0002;

        opts.derOutputs = { 'Loss1', 0, ...
                            'Loss2', 0, ...
                            'Loss3', 1, ...
                            'Loss4', 0, ...
                            'Loss5', 0, ...
                            'objective', 0};

        net.addLayer('objective', ...
          SegmentationLoss('loss', 'softmaxlog', 'inputNum', 6), ...
          {'x53', 'x55', 'x57', 'prediction', 'x49', 'x51', 'label'}, {'Loss3', 'Loss2', 'Loss1', 'objective', 'Loss5', 'Loss4'}) ;

        net.addLayer('accuracy', ...
          SegmentationAccuracy('numGroups', opts.numGroups), ...
          {'x53', 'label'}, 'accuracy') ;

      case 101

        net.params(55).learningRate = 0.0001;
        net.params(56).learningRate = 0.0002;
        net.params(43).learningRate = 0.0001;
        net.params(44).learningRate = 0.0002;

        opts.derOutputs = { 'Loss1', 0, ...
                            'Loss2', 0, ...
                            'Loss3', 0, ...
                            'Loss4', 1, ...
                            'Loss5', 0, ...
                            'objective', 0};

        net.addLayer('objective', ...
          SegmentationLoss('loss', 'softmaxlog', 'inputNum', 6), ...
          {'x51', 'x53', 'x55', 'x57', 'prediction', 'x49', 'label'}, {'Loss4', 'Loss3', 'Loss2', 'Loss1', 'objective', 'Loss5'}) ;

        net.addLayer('accuracy', ...
          SegmentationAccuracy('numGroups', opts.numGroups), ...
          {'x51', 'label'}, 'accuracy') ;

      case 51

        net.params(47).learningRate = 0.0001;
        net.params(48).learningRate = 0.0002;
        net.params(51).learningRate = 0.0001;
        net.params(52).learningRate = 0.0002;
        net.params(53).learningRate = 0.0001;
        net.params(54).learningRate = 0.0002;

        opts.derOutputs = { 'Loss1', 0, ...
                            'Loss2', 0, ...
                            'Loss3', 0, ...
                            'Loss4', 0, ...
                            'Loss5', 1, ...
                            'objective', 0};

        net.addLayer('objective', ...
          SegmentationLoss('loss', 'softmaxlog', 'inputNum', 6), ...
          {'x49', 'x51', 'x53', 'x55', 'x57', 'prediction', 'label'}, {'Loss5', 'Loss4', 'Loss3', 'Loss2', 'Loss1', 'objective'}) ;

        net.addLayer('accuracy', ...
          SegmentationAccuracy('numGroups', opts.numGroups), ...
          {'x49', 'label'}, 'accuracy') ;

      case 1

        opts.derOutputs = { 'Loss1', 0, ...
                            'Loss2', 0, ...
                            'Loss3', 0, ...
                            'Loss4', 0, ...
                            'Loss5', 0, ...
                            'objective', 1};

        net.addLayer('objective', ...
          SegmentationLoss('loss', 'softmaxlog', 'inputNum', 6), ...
          {'prediction', 'x49', 'x51', 'x53', 'x55', 'x57', 'label'}, {'objective', 'Loss5', 'Loss4', 'Loss3', 'Loss2', 'Loss1'}) ;

        net.addLayer('accuracy', ...
          SegmentationAccuracy('numGroups', opts.numGroups), ...
          {'prediction', 'label'}, 'accuracy') ;
      otherwise
        break;
    end
  end

  % train one epoch
  state.epoch = epoch ;
  state.learningRate = opts.learningRate(min(epoch, numel(opts.learningRate))) ;
  state.train = opts.train(randperm(numel(opts.train))) ; % shuffle
  state.val = opts.val ;
  state.imdb = imdb ;

  if numGpus <= 1
    stats.train(epoch) = process_epoch(net, state, opts, 'train') ;
    stats.val(epoch) = process_epoch(net, state, opts, 'val') ;
  else
    savedNet = net.saveobj() ;
    spmd
      net_ = dagnn.DagNN.loadobj(savedNet) ;
      stats_.train = process_epoch(net_, state, opts, 'train') ;
      stats_.val = process_epoch(net_, state, opts, 'val') ;
      if labindex == 1, savedNet_ = net_.saveobj() ; end
    end
    net = dagnn.DagNN.loadobj(savedNet_{1}) ;
    stats__ = accumulateStats(stats_) ;
    stats.train(epoch) = stats__.train ;
    stats.val(epoch) = stats__.val ;
    clear net_ stats_ stats__ savedNet_ ;
  end

  if ~evaluateMode && mod(epoch, 5) == 0
    saveState(modelPath(epoch), net, stats, opts) ;
  end

  if opts.plotStatistics
    delete(modelFigPath);
    figure(1) ; clf ;
    plots = setdiff(...
      cat(2,...
      fieldnames(stats.train)', ...
      fieldnames(stats.val)'), {'num', 'time'}) ;

    plots = [plots(1:end - 2) plots(end) plots(end - 1)];
    col = 1;

    for p = plots
      p = char(p) ;
      values = zeros(0, epoch) ;
      leg = {} ;
      for f = {'train', 'val'}
        f = char(f) ;
        if isfield(stats.(f), p)
          tmp = [stats.(f).(p)] ;
          values(end+1,:) = tmp(1,:)' ;
          leg{end+1} = f ;
        end
      end
      subplot(1,2,col) ;
      plot(1:epoch, values','o-') ;
      xlabel('epoch') ;
      title(p) ;
      legend(leg{:}) ;
      grid on ;
      drawnow ;
      
      if ~mod(col, 2) || strcmp(cellstr(p), cellstr(plots(end)))
        print(1, modelFigPath, '-dpsc', '-append') ;
      end
      
      col = mod(col, 2) + 1;
      
    end
  end
end

% -------------------------------------------------------------------------
function stats = process_epoch(net, state, opts, mode)
% -------------------------------------------------------------------------

if strcmp(mode,'train')
  state.momentum = num2cell(zeros(1, numel(net.params))) ;
end

numGpus = numel(opts.gpus) ;
if numGpus >= 1
  net.move('gpu') ;
  if strcmp(mode,'train')
    state.momentum = cellfun(@gpuArray,state.momentum,'UniformOutput',false) ;
  end
end
if numGpus > 1
  mmap = map_gradients(opts.memoryMapFile, net, numGpus) ;
else
  mmap = [] ;
end

stats.time = 0 ;
stats.num = 0 ;
subset = state.(mode) ;
start = tic ;
num = 0 ;

for t=1:opts.batchSize:numel(subset)
  batchSize = min(opts.batchSize, numel(subset) - t + 1) ;

  for s=1:opts.numSubBatches
    % get this image batch and prefetch the next
    batchStart = t + (labindex-1) + (s-1) * numlabs ;
    batchEnd = min(t+opts.batchSize-1, numel(subset)) ;
    batch = subset(batchStart : opts.numSubBatches * numlabs : batchEnd) ;
    num = num + numel(batch) ;
    if numel(batch) == 0, continue ; end

    inputs = state.getBatch(state.imdb, batch) ;

    if opts.prefetch
      if s == opts.numSubBatches
        batchStart = t + (labindex-1) + opts.batchSize ;
        batchEnd = min(t+2*opts.batchSize-1, numel(subset)) ;
      else
        batchStart = batchStart + numlabs ;
      end
      nextBatch = subset(batchStart : opts.numSubBatches * numlabs : batchEnd) ;
      state.getBatch(state.imdb, nextBatch) ;
    end

    if strcmp(mode, 'train')
      net.mode = 'normal' ;
      net.accumulateParamDers = (s ~= 01) ;
      net.eval(inputs, opts.derOutputs) ;
    else
      net.mode = 'test' ;
      net.eval(inputs) ;
    end
  end

  % extract learning stats
  stats = opts.extractStatsFn(net) ;

  % accumulate gradient
  if strcmp(mode, 'train')
    if ~isempty(mmap)
      write_gradients(mmap, net) ;
      labBarrier() ;
    end
    state = accumulate_gradients(state, net, opts, batchSize, mmap) ;
  end

  % print learning statistics
  time = toc(start) ;
  stats.num = num ;
  stats.time = toc(start) ;

  fprintf('%s: epoch %02d: %3d/%3d: %.1f Hz', ...
    mode, ...
    state.epoch, ...
    fix(t/opts.batchSize)+1, ceil(numel(subset)/opts.batchSize), ...
    stats.num/stats.time * max(numGpus, 1)) ;
  for f = setdiff(fieldnames(stats)', {'num', 'time'})
    f = char(f) ;
    fprintf(' %s:', f) ;
    fprintf(' %.3f', stats.(f)) ;
  end
  fprintf('\n') ;
end

net.reset() ;
net.move('cpu') ;

% -------------------------------------------------------------------------
function state = accumulate_gradients(state, net, opts, batchSize, mmap)
% -------------------------------------------------------------------------
for p=1:numel(net.params)

  % bring in gradients from other GPUs if any
  if ~isempty(mmap)
    numGpus = numel(mmap.Data) ;
    tmp = zeros(size(mmap.Data(labindex).(net.params(p).name)), 'single') ;
    for g = setdiff(1:numGpus, labindex)
      tmp = tmp + mmap.Data(g).(net.params(p).name) ;
    end
    net.params(p).der = net.params(p).der + tmp ;
  else
    numGpus = 1 ;
  end

  switch net.params(p).trainMethod

    case 'average' % mainly for batch normalization
      thisLR = net.params(p).learningRate ;
      net.params(p).value = ...
          (1 - thisLR) * net.params(p).value + ...
          (thisLR/batchSize/net.params(p).fanout) * net.params(p).der ;

    case 'gradient'
      thisDecay = opts.weightDecay * net.params(p).weightDecay ;
      thisLR = state.learningRate * net.params(p).learningRate ;
      state.momentum{p} = opts.momentum * state.momentum{p} ...
        - thisDecay * net.params(p).value ...
        - (1 / batchSize) * net.params(p).der ;
      net.params(p).value = net.params(p).value + thisLR * state.momentum{p} ;

    case 'otherwise'
      error('Unknown training method ''%s'' for parameter ''%s''.', ...
        net.params(p).trainMethod, ...
        net.params(p).name) ;
  end
end

% -------------------------------------------------------------------------
function mmap = map_gradients(fname, net, numGpus)
% -------------------------------------------------------------------------
format = {} ;
for i=1:numel(net.params)
  format(end+1,1:3) = {'single', size(net.params(i).value), net.params(i).name} ;
end
format(end+1,1:3) = {'double', [3 1], 'errors'} ;
if ~exist(fname) && (labindex == 1)
  f = fopen(fname,'wb') ;
  for g=1:numGpus
    for i=1:size(format,1)
      fwrite(f,zeros(format{i,2},format{i,1}),format{i,1}) ;
    end
  end
  fclose(f) ;
end
labBarrier() ;
mmap = memmapfile(fname, 'Format', format, 'Repeat', numGpus, 'Writable', true) ;

% -------------------------------------------------------------------------
function write_gradients(mmap, net)
% -------------------------------------------------------------------------
for i=1:numel(net.params)
  mmap.Data(labindex).(net.params(i).name) = gather(net.params(i).der) ;
end

% -------------------------------------------------------------------------
function stats = accumulateStats(stats_)
% -------------------------------------------------------------------------

stats = struct() ;

for s = {'train', 'val'}
  s = char(s) ;
  total = 0 ;

  for g = 1:numel(stats_)
    stats__ = stats_{g} ;
    num__ = stats__.(s).num ;
    total = total + num__ ;

    for f = setdiff(fieldnames(stats__.(s))', 'num')
      f = char(f) ;

      if g == 1
        stats.(s).(f) = 0 ;
      end
      stats.(s).(f) = stats.(s).(f) + stats__.(s).(f) * num__ ;

      if g == numel(stats_)
        stats.(s).(f) = stats.(s).(f) / total ;
      end
    end
  end
  stats.(s).num = total ;
end

% -------------------------------------------------------------------------
function stats = extractStats(net)
% -------------------------------------------------------------------------
sel = find(cellfun(@(x) isa(x,'dagnn.Loss'), {net.layers.block})) ;
stats = struct() ;
for i = 1:numel(sel)
  stats.(net.layers(sel(i)).name) = net.layers(sel(i)).block.average ;
  
% multiple loss outputs, add by yangtao 2017.06.03
%   for j = 1 : numel(net.layers(sel(i)).outputs)
%       stats.(net.layers(sel(i)).outputs{j}) = net.layers(sel(i)).block.average(j);
%   end
end

% -------------------------------------------------------------------------
function saveState(fileName, net, stats, opts)
% -------------------------------------------------------------------------
net_ = net ;
net = net_.saveobj() ;
save(fileName, 'net', 'stats', 'opts') ;

% -------------------------------------------------------------------------
function [net, stats] = loadState(fileName)
% -------------------------------------------------------------------------
load(fileName, 'net', 'stats') ;
net = dagnn.DagNN.loadobj(net) ;

% -------------------------------------------------------------------------
function epoch = findLastCheckpoint(modelDir)
% -------------------------------------------------------------------------
list = dir(fullfile(modelDir, 'net-epoch-*.mat')) ;
tokens = regexp({list.name}, 'net-epoch-([\d]+).mat', 'tokens') ;
epoch = cellfun(@(x) sscanf(x{1}{1}, '%d'), tokens) ;
epoch = max([epoch 0]) ;